import sys
import pygame
import random
import time
import path




from pygame.locals import *
from const import *
from sprite import *
from vector import *

def font_surface(text,size,colour):
    font = pygame.font.get_default_font()
    font_layer = pygame.font.Font(font,size)
    font_surface = font_layer.render(text,True,colour)
    return font_surface


def title_screen():
    pygame.init()
    pygame.mixer.init()

    pygame.mixer.music.load(path.filepath('bg.ogg'))
    pygame.mixer.music.set_volume(0.4)
    pygame.mixer.music.play(-1)
    
    screen = pygame.display.set_mode((WIDTH,HEIGHT))
    pygame.mouse.set_visible(False)
    pygame.display.set_caption('The Aftermath')

    bg_image = pygame.image.load(path.filepath('title_screen.png')).convert_alpha()
    spider_image = pygame.image.load(path.filepath('spider.png')).convert_alpha()
    player_image = pygame.image.load(path.filepath('player.png')).convert_alpha()
    bullet_image = pygame.image.load(path.filepath('bullet.png')).convert_alpha()
    cursor = pygame.image.load(path.filepath('zhunxing.png')).convert_alpha()

    spider_list = pygame.sprite.Group()
    player_list = pygame.sprite.Group()
    bullet_list = pygame.sprite.Group()
    all_sprite = pygame.sprite.Group()

    cursor = Entity('cursor',cursor)
    all_sprite.add(cursor)
    player = Player(player_image)
    player_list.add(player)
    all_sprite.add(player)
    spider = Spider(spider_image)
    spider.location = Vector(600,200)
    spider.destination = Vector(600,200)
    spider_list.add(spider)
    all_sprite.add(spider)

    text1 = 'Click to shoot!'
    text_surface1 = font_surface(text1,20,DARKGREEN)
    text2 = 'W S A D to move!'
    text_surface2 = font_surface(text2,20,DARKGREEN)
    text3 = 'Shoot the spider to start the game'
    text_surface3 = font_surface(text3,20,DARKGREEN)

    clock = pygame.time.Clock()    

    while True:
        screen.fill(BG)
        screen.blit(bg_image,(0,0))
        cursor.rect.center = pygame.mouse.get_pos()
        cursor_position = Vector(*cursor.rect.center)
        screen.blit(text_surface1,cursor.rect.center)
        screen.blit(text_surface2,player.rect.center)
        screen.blit(text_surface3,(50,550))

        for event in pygame.event.get():
            if event.type == QUIT:
                pygame.quit()
                sys.exit()
            elif event.type == MOUSEBUTTONDOWN:
                    bullet_sound = pygame.mixer.Sound(path.filepath('bullet.ogg'))
                    bullet_sound.play()
                    bullet = Bullet(bullet_image)
                    bullet.direction = cursor_position - Vector(*player.rect.center)
                    bullet.direction.normalize()
                    bullet.location = Vector(*player.rect.center)+bullet.direction*30
                    bullet_list.add(bullet)
                    all_sprite.add(bullet)

        pressed_keys = pygame.key.get_pressed()
        player.direction = Vector(0,0)
        if pressed_keys[K_a]:
            player.direction.x = -1
        elif pressed_keys[K_d]:
            player.direction.x = +1
        if pressed_keys[K_w]:
            player.direction.y = -1
        elif pressed_keys[K_s]:
            player.direction.y = +1
        player.direction.normalize()
        player.location += player.direction * player.speed

        all_sprite.draw(screen)
        all_sprite.update()

        if pygame.sprite.spritecollide(spider,bullet_list,True):
            pygame.mixer.music.stop()
            return

        clock.tick(FPS)
        pygame.display.update()




def run():
    title_screen()
    pygame.init()
    pygame.mixer.init()
    screen = pygame.display.set_mode((WIDTH,HEIGHT))
    pygame.mouse.set_visible(False)

    health_image = pygame.image.load(path.filepath('health.png')).convert_alpha()
    cursor = pygame.image.load(path.filepath('zhunxing.png')).convert_alpha()
    spider_image = pygame.image.load(path.filepath('spider.png')).convert_alpha()
    player_image = pygame.image.load(path.filepath('player.png')).convert_alpha()
    bullet_image = pygame.image.load(path.filepath('bullet.png')).convert_alpha()

    spider_list = pygame.sprite.Group()
    player_list = pygame.sprite.Group()
    bullet_list = pygame.sprite.Group()
    all_sprite = pygame.sprite.Group()
    health_list = pygame.sprite.Group()

    cursor = Entity('cursor',cursor)
    all_sprite.add(cursor)
    
    player = Player(player_image)
    player_list.add(player)
    all_sprite.add(player)

    for i in range(20):
        spider = Spider(spider_image)
        spider_list.add(spider)
        all_sprite.add(spider)

    wave = 1
    score = 0

    clock = pygame.time.Clock()

    while True:
        t = time.clock() % 30
        if t > 19.98 and t < 20:
            for i in range(20 + wave * 5):
                spider = Spider(spider_image)
                spider.speed = wave/2
                spider_list.add(spider)
                all_sprite.add(spider)
            wave += 1
        elif t >29.98:
            emerge = Emerge(health_image)
            health_list.add(emerge)
            all_sprite.add(emerge)

        cursor.rect.center = pygame.mouse.get_pos()
        cursor_position = Vector(*cursor.rect.center)
        
        for event in pygame.event.get():
            if event.type == QUIT:
                pygame.quit()
                sys.exit()
            elif event.type == MOUSEBUTTONDOWN:
                bullet_sound = pygame.mixer.Sound(path.filepath('bullet.ogg'))
                bullet_sound.play()

                bullet = Bullet(bullet_image)
                bullet.direction = cursor_position - Vector(*player.rect.center)
                bullet.direction.normalize()
                bullet.location = Vector(*player.rect.center)+bullet.direction*30
                bullet_list.add(bullet)
                all_sprite.add(bullet)
                if player.alive == True:
                    score += 10
                if player.alive == False:
                    pass

        pressed_keys = pygame.key.get_pressed()
        if pressed_keys[K_ESCAPE]:
            pygame.quit()
            sys.exit()
        
        player.direction = Vector(0,0)
        if pressed_keys[K_a]:
            player.direction.x = -1
        elif pressed_keys[K_d]:
            player.direction.x = +1
        if pressed_keys[K_w]:
            player.direction.y = -1
        elif pressed_keys[K_s]:
            player.direction.y = +1
        player.direction.normalize()
        player.location += player.direction * player.speed

        for spider in spider_list:
            spider.destination = player.location

        screen.fill(BG)
        all_sprite.draw(screen)

        #
        # Here is the biggist bug i have find until now,can you help me to fix it?
        #
        spider_hit_list = pygame.sprite.groupcollide(spider_list,bullet_list,False,True)
        for spider in spider_hit_list.keys():
            score += 20
            pygame.draw.rect(spider.image,RED,(0,0,40-spider.health,2))
            spider.hit()

        bullet_hit_player = pygame.sprite.spritecollide(player,bullet_list,True)
        if bullet_hit_player:
            player.hit2()

        # The game speed is quit quick, so the effect is not obvious
        attacker = None
        attacker = pygame.sprite.spritecollideany(player,spider_list)
        if attacker != None:
            if pygame.sprite.collide_rect_ratio(0.5)(player,attacker):
                player.hit2()
                if attacker.rect.x < player.rect.x:
                    attacker.rect.x -= 20
                elif attacker.rect.x >= player.rect.x:
                    attacker.rect.x += 20
            else:
                attcker = None

        player_get_health = pygame.sprite.spritecollide(player,health_list,True)
        if player_get_health and player.health <100:
            get_health = pygame.mixer.Sound(path.filepath('health.ogg'))
            get_health.play()
            player.health += 30
            if player.health >= 100:
                player.health =100

        pygame.draw.rect(screen,GREEN,(20,480,10,100))
        if player.health < 100:
            pygame.draw.rect(screen,RED,(20,480,10,100-player.health))
        all_sprite.update()

        text1 = 'Socre : ' + str(score)
        text_surface1 = font_surface(text1,30,DARKGREEN)
        screen.blit(text_surface1,(10,10))

        if score <= RECORD:
            text2 = 'Record : '+ str(RECORD)
            text_surface2 = font_surface(text2,30,DARKGREEN)
            t_w = text_surface2.get_width()
            screen.blit(text_surface2,(790-t_w,10))

        elif score > RECORD:
            text2 = 'Record : '+ str(score)
            text_surface2 = font_surface(text2,30,RED)
            t_w = text_surface2.get_width()
            screen.blit(text_surface2,(790-t_w,10))
            text = 'GREAT! NEW RECORD!'
            text_surface = font_surface(text,20,RED)
            t_w = text_surface.get_width()
            screen.blit(text_surface,(790-t_w,50))


        if player.alive == False:
            
            all_sprite.empty()
            if score > RECORD:
                high_score = path.load('record.txt','w')
                high_score.write(str(score))
                high_score.close()
            
            text3 = 'GAME OVER'
            text_surface3 = font_surface(text3,60,RED)
            t_w = text_surface3.get_width()
            screen.blit(text_surface3,(400-t_w/2,200))

            text4 = 'THANKS  FOR  PLAYING  ^v^'
            text_surface4 = font_surface(text4,30,RED)
            t_w = text_surface4.get_width()
            screen.blit(text_surface4,(400-t_w/2,350))

        clock.tick(FPS)
        pygame.display.update()

if __name__ == '__main__':
    run()
