import pygame
import random
import math
import time
import path

from pygame.locals import *
from const import *
from vector import *



class Entity(pygame.sprite.Sprite):
    def __init__(self,name,image,speed=0):
        super().__init__()
        self.name = name
        self.image = image
        self.speed = speed
        self.rect = self.image.get_rect()
        self.location = Vector(400,300)
        self.destination = Vector(200,200)
        self.direction = Vector(1,1)

class Emerge(Entity):
    def __init__(self,image):
        Entity.__init__(self,'emerge',image)
        self.location = Vector(random.randint(100,700),random.randint(100,500))

    def update(self):
        self.rect.x, self.rect.y = self.location

class Bullet(Entity):
    def __init__(self,image):
        Entity.__init__(self,'bullet',image,5)

    def update(self):
        self.location += self.direction * self.speed
        self.rect.x, self.rect.y = self.location
        if self.rect.x > WIDTH-self.image.get_width():
            self.direction = Vector(-1,random.random())
        elif self.rect.x <self.image.get_width():
            self.direction = Vector(1,random.random())
        elif self.rect.y > HEIGHT - self.image.get_height():
            self.direction = Vector(random.random(),-1)
        elif self.rect.y < self.image.get_height():
            self.direction = Vector(random.random(),1)

class Player(Entity):
    def __init__(self,image):
        Entity.__init__(self,'player',image,3)
        self.health = 100
        self.alive = True

    def hit2(self):
        hit2_sound=pygame.mixer.Sound(path.filepath('bullet_hit_player.ogg'))
        hit2_sound.play()
        self.health -= 10
        if self.health <= 0:
            self.dead()

    def dead(self):
        player_dead_sound = pygame.mixer.Sound(path.filepath('player_alive.ogg'))
        player_dead_sound.play()
        self.alive = False
        self.kill()

    def update(self):
        self.rect.x, self.rect.y = self.location
        if self.rect.x > WIDTH-self.image.get_width():
            self.rect.x = WIDTH-self.image.get_width()
        if self.rect.x < 0:
            self.rect.x = 0
        if self.rect.y > HEIGHT - self.image.get_height():
            self.rect.y = HEIGHT - self.image.get_height()
        if self.rect.y < 0:
            self.rect.y = 0

class Spider(Entity):
    def __init__(self,image):
        Entity.__init__(self,'spider',image,1)
        self.image.fill(GREEN,(0,0,40,2))
        self.image.set_colorkey(RED)
        self.alive = True
        self.health = 40
        seed = random.randint(1,4)
        if seed == 1:
            self.location = Vector(random.randint(0,WIDTH),-random.randint(50,200))
        if seed == 2:
            self.location = Vector(random.randint(0,WIDTH),random.randint(HEIGHT+50,HEIGHT+200))
        if seed == 3:
            self.location = Vector(-random.randint(50,200),random.randint(0,HEIGHT))
        if seed == 4:
            self.location = Vector(random.randint(WIDTH + 50,WIDTH +200),random.randint(0,HEIGHT))
       
        self.rect.x, self.rect.y = self.location
        self.destination = Vector(random.randint(0,WIDTH),random.randint(0,HEIGHT))
        
    def hit(self):
        self.health -= 10

        if self.health <= 0:
            self.dead()

    def dead(self):
        self.alive = False
        self.kill()

    def update(self):
        self.direction =(self.destination - self.location).normalize()
        self.location += self.direction*self.speed
        self.rect.x, self.rect.y = self.location
        
        
        
        
