import math

class Vector():

    def __init__(self,x=0.0,y=0.0):
        self.x = x
        self.y = y

    def __iter__(self):
        
        return iter([self.x, self.y])

    def __str__(self):
        return '('+'{:.3f}'.format(self.x)+','+'{:.3f}'.format(self.y)+')'

    @classmethod
    def from_points(self,P1,P2):
        return Vector(P2[0]-P1[0],P2[1]-P1[1])

    def dot_product(self,rhs):
        return self.x*rhs.x+self.y*rhs.y

    def included_angle(self,rhs):
        return 360-math.degrees(math.acos(self.dot_product(rhs)/self.magnitude()/rhs.magnitude()))

    def magnitude(self):
        return math.sqrt(self.x ** 2 + self.y**2)

    def normalize(self):
        try:
            magnitude = self.magnitude()
            self.x /= magnitude
            self.y /= magnitude
        except ZeroDivisionError:
            self.x = self.x
            self.y = self.y
        return Vector(self.x,self.y)

    def __neg__(self):
        return Vector(-self.x,-self.y)

    def __sub__(self,rhs):
        return Vector(self.x-rhs.x, self.y-rhs.y)

    def __add__(self,rhs):
        return Vector(self.x+rhs.x, self.y+rhs.y)

    def __mul__(self,scalar):
        return Vector(self.x * scalar, self.y *scalar)



